@@ -18,6 +18,7 @@ import org.postgresql.core.v3.BatchedQuery;
 import org.postgresql.largeobject.LargeObject;
 import org.postgresql.largeobject.LargeObjectManager;
 import org.postgresql.util.ByteConverter;
+import org.postgresql.util.ByteStreamWriter;
 import org.postgresql.util.GT;
 import org.postgresql.util.HStoreConverter;
 import org.postgresql.util.PGBinaryObject;
@@ -183,10 +184,10 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
   public void setNull(int parameterIndex, int sqlType) throws SQLException {
     checkClosed();
 
-    if (parameterIndex < 1 || parameterIndex > preparedParameters.getInParameterCount()) {
+    if (parameterIndex < 1 || parameterIndex > preparedParameters.getParameterCount()) {
       throw new PSQLException(
         GT.tr("The column index is out of range: {0}, number of columns: {1}.",
-          parameterIndex, preparedParameters.getInParameterCount()),
+          parameterIndex, preparedParameters.getParameterCount()),
         PSQLState.INVALID_PARAMETER_VALUE);
     }
 
@@ -363,6 +364,10 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     preparedParameters.setBytea(parameterIndex, copy, 0, x.length);
   }
 
+  private void setByteStreamWriter(int parameterIndex, ByteStreamWriter x) throws SQLException {
+    preparedParameters.setBytea(parameterIndex, x);
+  }
+
   public void setDate(int parameterIndex, java.sql.Date x) throws SQLException {
     setDate(parameterIndex, x, null);
   }
@@ -930,6 +935,8 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       setDouble(parameterIndex, (Double) x);
     } else if (x instanceof byte[]) {
       setBytes(parameterIndex, (byte[]) x);
+    } else if (x instanceof ByteStreamWriter) {
+      setByteStreamWriter(parameterIndex, (ByteStreamWriter) x);
     } else if (x instanceof java.sql.Date) {
       setDate(parameterIndex, (java.sql.Date) x);
     } else if (x instanceof Time) {
