@@ -55,10 +55,7 @@ package org.postgresql.util;
  * @version 2.1
  */
 public class Base64 {
-
   /* ******** P U B L I C F I E L D S ******** */
-
-
   /**
    * No options specified. Value is zero.
    */
@@ -69,46 +66,37 @@ public class Base64 {
    */
   public static final int ENCODE = 1;
 
-
   /**
    * Specify decoding.
    */
   public static final int DECODE = 0;
 
-
   /**
    * Don't break lines when encoding (violates strict Base64 specification).
    */
   public static final int DONT_BREAK_LINES = 8;
 
-
   /* ******** P R I V A T E F I E L D S ******** */
-
-
   /**
    * Maximum line length (76) of Base64 output.
    */
   private static final int MAX_LINE_LENGTH = 76;
 
-
   /**
    * The equals sign (=) as a byte.
    */
   private static final byte EQUALS_SIGN = (byte) '=';
 
-
   /**
    * The new line character (\n) as a byte.
    */
   private static final byte NEW_LINE = (byte) '\n';
 
-
   /**
    * Preferred encoding.
    */
   private static final String PREFERRED_ENCODING = "UTF-8";
 
-
   /**
    * The 64 valid Base64 values.
    */
@@ -128,17 +116,16 @@ public class Base64 {
 
   /* Determine which ALPHABET to use. */
   static {
-    byte[] __bytes;
+    byte[] bytes;
     try {
-      __bytes = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
+      bytes = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
           .getBytes(PREFERRED_ENCODING);
     } catch (java.io.UnsupportedEncodingException use) {
-      __bytes = _NATIVE_ALPHABET; // Fall back to native encoding
+      bytes = _NATIVE_ALPHABET; // Fall back to native encoding
     }
-    ALPHABET = __bytes;
+    ALPHABET = bytes;
   }
 
-
   /**
    * Translates a Base64 value to either its 6-bit reconstruction value or a negative number
    * indicating some other meaning.
@@ -183,36 +170,13 @@ public class Base64 {
   private static final byte WHITE_SPACE_ENC = -5; // Indicates white space in encoding
   private static final byte EQUALS_SIGN_ENC = -1; // Indicates equals sign in encoding
 
-
   /**
    * Defeats instantiation.
    */
   private Base64() {
   }
 
-
   /* ******** E N C O D I N G M E T H O D S ******** */
-
-
-  /**
-   * Encodes up to the first three bytes of array <var>threeBytes</var> and returns a four-byte
-   * array in Base64 notation. The actual number of significant bytes in your array is given by
-   * <var>numSigBytes</var>. The array <var>threeBytes</var> needs only be as big as
-   * <var>numSigBytes</var>. Code can reuse a byte array by passing a four-byte array as
-   * <var>b4</var>.
-   *
-   * @param b4 A reusable byte array to reduce array instantiation
-   * @param threeBytes the array to convert
-   * @param numSigBytes the number of significant bytes in your array
-   * @return four byte array in Base64 notation.
-   * @since 1.5.1
-   */
-  private static byte[] encode3to4(byte[] b4, byte[] threeBytes, int numSigBytes) {
-    encode3to4(threeBytes, 0, numSigBytes, b4, 0);
-    return b4;
-  } // end encode3to4
-
-
   /**
    * Encodes up to three bytes of the array <var>source</var> and writes the resulting four Base64
    * bytes to <var>destination</var>. The source and destination arrays can be manipulated anywhere
@@ -284,7 +248,6 @@ public class Base64 {
     return encodeBytes(source, 0, source.length, NO_OPTIONS);
   } // end encodeBytes
 
-
   /**
    * <p>Encodes a byte array into Base64 notation.</p>
    *
@@ -311,7 +274,6 @@ public class Base64 {
     return encodeBytes(source, 0, source.length, options);
   } // end encodeBytes
 
-
   /**
    * Encodes a byte array into Base64 notation. Does not GZip-compress data.
    *
@@ -325,7 +287,6 @@ public class Base64 {
     return encodeBytes(source, off, len, NO_OPTIONS);
   } // end encodeBytes
 
-
   /**
    * <p>Encodes a byte array into Base64 notation.</p>
    *
@@ -383,22 +344,16 @@ public class Base64 {
         e += 4;
       } // end if: some padding needed
 
-
       // Return value according to relevant encoding.
       try {
         return new String(outBuff, 0, e, PREFERRED_ENCODING);
       } catch (java.io.UnsupportedEncodingException uue) {
         return new String(outBuff, 0, e);
       }
-
     }
-
   }
 
-
   /* ******** D E C O D I N G M E T H O D S ******** */
-
-
   /**
    * Decodes four bytes from array <var>source</var> and writes the resulting bytes (up to three of
    * them) to <var>destination</var>. The source and destination arrays can be manipulated anywhere
@@ -452,7 +407,6 @@ public class Base64 {
             | ((DECODABET[source[srcOffset + 2]] & 0xFF) << 6)
             | ((DECODABET[source[srcOffset + 3]] & 0xFF));
 
-
         destination[destOffset] = (byte) (outBuff >> 16);
         destination[destOffset + 1] = (byte) (outBuff >> 8);
         destination[destOffset + 2] = (byte) (outBuff);
@@ -468,7 +422,6 @@ public class Base64 {
     }
   } // end decodeToBytes
 
-
   /**
    * Very low-level access to decoding ASCII characters in the form of a byte array. Does not
    * support automatically gunzipping or any other "fancy" features.
@@ -520,7 +473,6 @@ public class Base64 {
     return out;
   } // end decode
 
-
   /**
    * Decodes data from Base64 notation, automatically detecting gzip-compressed data and
    * decompressing it.
