@@ -134,9 +134,9 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
          * If the argument to equals-method is also a wrapper, present the original unwrapped
          * connection to the underlying equals method.
          */
-        if (method.getName().equals("equals")) {
+        if (method.getName().equals("equals") && args.length == 1) {
           Object arg = args[0];
-          if (Proxy.isProxyClass(arg.getClass())) {
+          if (arg != null && Proxy.isProxyClass(arg.getClass())) {
             InvocationHandler h = Proxy.getInvocationHandler(arg);
             if (h instanceof ConnectionHandler) {
               // unwrap argument
@@ -408,7 +408,7 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
           }
           rs.close();
 
-          return l.toArray(new Xid[l.size()]);
+          return l.toArray(new Xid[0]);
         } finally {
           stmt.close();
         }
@@ -573,7 +573,7 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
       if (state != State.IDLE
           || conn.getTransactionState() != TransactionState.IDLE) {
         throw new PGXAException(
-            GT.tr("Not implemented: 2nd phase commit must be issued using an idle connection. commit xid={0}, currentXid={1}, state={2], transactionState={3}", xid, currentXid, state, conn.getTransactionState()),
+            GT.tr("Not implemented: 2nd phase commit must be issued using an idle connection. commit xid={0}, currentXid={1}, state={2}, transactionState={3}", xid, currentXid, state, conn.getTransactionState()),
             XAException.XAER_RMERR);
       }
 
