@@ -57,7 +57,6 @@ public class DatabaseMetaDataTest {
     TestUtil.createCompositeType(con, "custom", "i int", false);
     TestUtil.createCompositeType(con, "_custom", "f float", false);
 
-
     // 8.2 does not support arrays of composite types
     TestUtil.createTable(con, "customtable", "c1 custom, c2 _custom"
         + (TestUtil.haveMinimumServerVersion(con, ServerVersion.v8_3) ? ", c3 custom[], c4 _custom[]" : ""));
@@ -117,7 +116,6 @@ public class DatabaseMetaDataTest {
     TestUtil.dropDomain(con, "varbit2");
     TestUtil.dropDomain(con, "float83");
 
-
     TestUtil.closeDB(con);
   }
 
@@ -224,7 +222,6 @@ public class DatabaseMetaDataTest {
     TestUtil.createTable(con1, "ww",
         "m int not null, n int not null, constraint m_pkey primary key ( m, n ), constraint ww_m_fkey foreign key ( m, n ) references vv ( a, b )");
 
-
     DatabaseMetaData dbmd = con.getMetaData();
     assertNotNull(dbmd);
 
@@ -259,7 +256,6 @@ public class DatabaseMetaDataTest {
     }
     assertEquals(2, numRows);
 
-
     TestUtil.dropTable(con1, "vv");
     TestUtil.dropTable(con1, "ww");
     TestUtil.closeDB(con1);
@@ -365,7 +361,6 @@ public class DatabaseMetaDataTest {
             + "REFERENCES PERSON (FIRST_NAME, LAST_NAME) MATCH SIMPLE "
             + "ON UPDATE CASCADE ON DELETE CASCADE");
 
-
     DatabaseMetaData dbmd = con.getMetaData();
     assertNotNull(dbmd);
     ResultSet rs = dbmd.getImportedKeys(null, "", "person");
@@ -409,8 +404,6 @@ public class DatabaseMetaDataTest {
 
     TestUtil.dropTable(con1, "person");
     TestUtil.closeDB(con1);
-
-
   }
 
   @Test
@@ -424,7 +417,6 @@ public class DatabaseMetaDataTest {
             + "CONSTRAINT people FOREIGN KEY (people_id) references people(id),"
             + "constraint policy FOREIGN KEY (policy_id) references policy(id)");
 
-
     DatabaseMetaData dbmd = con.getMetaData();
     assertNotNull(dbmd);
 
@@ -467,7 +459,6 @@ public class DatabaseMetaDataTest {
 
     assertTrue(rs.getString("FK_NAME").startsWith("people"));
 
-
     TestUtil.dropTable(con1, "users");
     TestUtil.dropTable(con1, "people");
     TestUtil.dropTable(con1, "policy");
@@ -542,9 +533,7 @@ public class DatabaseMetaDataTest {
     assertTrue(rs.next());
     assertEquals("updated", rs.getString(4));
 
-
     rs.close();
-
   }
 
   @Test
@@ -669,6 +658,32 @@ public class DatabaseMetaDataTest {
     rs.close();
   }
 
+  /**
+   * Order defined at
+   * https://docs.oracle.com/javase/8/docs/api/java/sql/DatabaseMetaData.html#getIndexInfo-java.lang.String-java.lang.String-java.lang.String-boolean-boolean-
+   */
+  @Test
+  public void testIndexInfoColumnOrder() throws SQLException {
+    DatabaseMetaData dbmd = con.getMetaData();
+    assertNotNull(dbmd);
+    ResultSet rs = dbmd.getIndexInfo(null, null, "metadatatest", false, false);
+    assertEquals(rs.findColumn("TABLE_CAT"), 1);
+    assertEquals(rs.findColumn("TABLE_SCHEM"), 2);
+    assertEquals(rs.findColumn("TABLE_NAME"), 3);
+    assertEquals(rs.findColumn("NON_UNIQUE"), 4);
+    assertEquals(rs.findColumn("INDEX_QUALIFIER"), 5);
+    assertEquals(rs.findColumn("INDEX_NAME"), 6);
+    assertEquals(rs.findColumn("TYPE"), 7);
+    assertEquals(rs.findColumn("ORDINAL_POSITION"), 8);
+    assertEquals(rs.findColumn("COLUMN_NAME"), 9);
+    assertEquals(rs.findColumn("ASC_OR_DESC"), 10);
+    assertEquals(rs.findColumn("CARDINALITY"), 11);
+    assertEquals(rs.findColumn("PAGES"), 12);
+    assertEquals(rs.findColumn("FILTER_CONDITION"), 13);
+
+    rs.close();
+  }
+
   @Test
   public void testNotNullDomainColumn() throws SQLException {
     DatabaseMetaData dbmd = con.getMetaData();
@@ -716,7 +731,6 @@ public class DatabaseMetaDataTest {
     assertEquals("id", rs.getString("COLUMN_NAME"));
     assertEquals("A", rs.getString("ASC_OR_DESC"));
 
-
     assertTrue(rs.next());
     assertEquals("idx_a_d", rs.getString("INDEX_NAME"));
     assertEquals("quest", rs.getString("COLUMN_NAME"));
@@ -1042,7 +1056,6 @@ public class DatabaseMetaDataTest {
       assertEquals("data type", Types.DISTINCT, dataType);
       assertEquals("type name ", "testint8", typeName);
       assertEquals("remarks", "jdbc123", remarks);
-
     } finally {
       try {
         Statement stmt = con.createStatement();
@@ -1075,7 +1088,6 @@ public class DatabaseMetaDataTest {
       assertEquals("data type", Types.DISTINCT, dataType);
       assertEquals("type name ", "testint8", typeName);
       assertEquals("remarks", "jdbc123", remarks);
-
     } finally {
       try {
         Statement stmt = con.createStatement();
@@ -1107,7 +1119,6 @@ public class DatabaseMetaDataTest {
       assertEquals("data type", Types.DISTINCT, dataType);
       assertEquals("type name ", "testint8", typeName);
       assertEquals("remarks", "jdbc123", remarks);
-
     } finally {
       try {
         Statement stmt = con.createStatement();
@@ -1137,7 +1148,6 @@ public class DatabaseMetaDataTest {
       assertTrue("base type", rs.wasNull());
       assertEquals("data type", Types.STRUCT, dataType);
       assertEquals("type name ", "testint8", typeName);
-
     } finally {
       try {
         Statement stmt = con.createStatement();
@@ -1206,7 +1216,6 @@ public class DatabaseMetaDataTest {
     for (String typeName : stringTypeList) {
       assertTrue(types.contains(typeName));
     }
-
   }
 
   @Test
@@ -1296,7 +1305,6 @@ public class DatabaseMetaDataTest {
         }
       }
     }
-
   }
 
   @Test
@@ -1432,5 +1440,4 @@ public class DatabaseMetaDataTest {
 
     rs.close();
   }
-
 }
