@@ -10,7 +10,9 @@ import org.junit.runners.Suite;
 import org.junit.runners.Suite.SuiteClasses;
 
 @RunWith(Suite.class)
-@SuiteClasses({PGDataSourceFactoryTest.class})
+@SuiteClasses({
+    PGDataSourceFactoryTest.class,
+})
 public class OsgiTestSuite {
 
 }
