@@ -9,6 +9,8 @@ import org.junit.runner.RunWith;
 import org.junit.runners.Suite;
 
 @RunWith(Suite.class)
-@Suite.SuiteClasses(XADataSourceTest.class)
+@Suite.SuiteClasses({
+    XADataSourceTest.class,
+})
 public class XATestSuite {
 }
